﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;


namespace CM1620_Terminal
{
    public class SmartLink
    {
        
        //---------------------------------------------------------------------------
        protected enum CommCmd
        {
            TestLinker,
            LoganDevice,
            QueryDevice,
            AppUpdate,
            EraseApp,
            WriteApp,
            AppCheckSum,
            RebootToApp,
            WriteAppData,
            AppDataCheckSum,
            DeviceRename,
            BatteryStatus,
            BatteryVoltage,
            BatteryIR,
            Topology,
            StatusAll,
            SL0,
            Stop,
            Task,
            Status,
            UserTask,
            Recover,
            Logout,
            Help,
            VoltageCalibration,
            Reboot,

        };
        //---------------------------------------------------------------------------
        public enum CommState
        {
            csStart,
            csErr,
        };
        //---------------------------------------------------------------------------
        const char CHAR_PREFIX = '#';
        const char CHAR_SUFFIX = '\r';
        const char CHAR_NEWLINE = '\n';
        const char STRING_DELIM = ' ';
        const char STRING_TIPS = '?';


        //Loop query command list variable declaration
        private ArrayList checkInformationList = new ArrayList();
        private int checkInformationListIndex = 0;
        private char[] checkInformationListIndexData = new char[] { };


        //Common instruction constant declaration
        private readonly char[] hello = new char[] { '#', 'h', 'e', 'l', 'l', 'o', '\n', '\r' };                           //Equipment online query
        private readonly char[] login = new char[] { '#', 'l', 'o', 'g', 'i', 'n', ' ', 'n', 'u', 'l', 'l', '\n', '\r' };  //Establish a secure link
        private readonly char[] task = new char[] { '#', 't', 'a', 's', 'k', '\n', '\r' };                                 //Task query
        private readonly char[] status = new char[] { '#', 's', 't', 'a', 't', 'u', 's', '\n', '\r' };                     //Status query
        private readonly char[] recover = new char[] { '#', 'r', 'e', 'c', 'o', 'v', 'e', 'r', '\n', '\r' };               //Restore command
        private readonly char[] stop = new char[] { '#', 's', 't', 'o', 'p', '\n', '\r' };                                 //Stop task
        private readonly char[] logout = new char[] { '#', 'l', 'o', 'g', 'o', 'u', 't', '\n', '\r' };                     //Terminate the communication link
        private readonly char[] help = new char[] { '#', 'h', 'e', 'l', 'p', '\n', '\r' };                                 //help
        private readonly char[] reboot = new char[] { '#','r','e','b','o','o','t', '\n', '\r' };

        //Device information variable declaration
        public List<string> deviceName = new List<string> { };
        public List<string> AP = new List<string> { };
        public List<string> BT = new List<string> { };
        public List<string> HW = new List<string> { };


        public char[] userTaskData = new char[] { };                             //User instructions
        public string printInformationData;                                      // Print debugging instructions


        //Task query variable declaration
        public List<string> userdeviceNameList = new List<string> { };           
        public List<string> userTaskTypeList = new List<string> { };            
        public List<string> userBatteryTypeList = new List<string> { };         
        public List<string> userVoltageList = new List<string> { };
        public List<string> userCellsList = new List<string> { };
        public List<string> userBatteryCapacityList = new List<string> { };
        public List<string> userCurrentList = new List<string> { };
        public List<string> userBalanceList = new List<string> { };


        public string batteryType = "";
        public string cells = "";
        public string batteryCapacity = "";

        public string loginOK = "";

        public string moduleName = "";
        public int numberModules = 1;
        public string inputVoltage = "";
        public string outputVoltage = "";
        public string temperature = "";
        public string battGO = "";
        public string balance = "";
        public string errorCode = "";
        public string currentTaskType = "";

        public int quantityModule = 0;


        //State query information variable declaration
        public List<string> moduleNameList = new List<string> { };
        public List<string> numberModulesList = new List<string> { };
        public List<string> inputVoltageList = new List<string> { };
        public List<string> outputVoltageList = new List<string> { };
        public List<string> temperatureList = new List<string> { };
        public List<string> battGOList = new List<string> { };
        public List<string> batteryPercentage = new List<string> { };
        public List<string> balanceList = new List<string> { };
        public List<string> errorCodeList = new List<string> { };
        public List<string> currentTaskTypeList = new List<string> { };
        public ArrayList balancePortVoltageList = new ArrayList();
        public ArrayList balanceInternalResistanceList = new ArrayList();
        public List<string> taskCurrentList = new List<string> { };
        public List<string> inputPowerList = new List<string> { };
        public List<double> actualOutputCurrentList = new List<double> { };
        public double actualOutputCurrent = 0;
        public List<int> currentCapacityList = new List<int> { };
        public List<string> taskTimeList = new List<string> { };
        public string[] balancePortVoltage = new string[] { };
        public string[] balanceInternalResistance = new string[] { };
        public List<string> chargeErrorCodeList = new List<string> { };


        private const int BUFFER_SIZE = 512;
        public char[] sendDataBuffer = new char[BUFFER_SIZE];
        public int sendDataBufferLength = 0;
        public char[] receiveDataBuffer = new char[BUFFER_SIZE];
        private int ComTimeout;

        protected bool SmLnkPkgMsg = false;
        protected CommCmd curCmd = CommCmd.TestLinker;
        public CommState commState;
        




        //---------------------------------------------------------------------------
        public SmartLink(int timeout)
        {
            ComTimeout = timeout;
        }
        //---------------------------------------------------------------------------
        protected virtual bool Open(string dev_path) { return false; }
        //---------------------------------------------------------------------------
        public virtual void Close() { }
        //---------------------------------------------------------------------------
        protected virtual void WriteMssage() { }
        //---------------------------------------------------------------------------
        public virtual bool SearchSerialPortDevice() { return false; }
        //---------------------------------------------------------------------------
        public virtual bool LoganDvice() { return false; }
        //---------------------------------------------------------------------------
        protected virtual void WaiteForDeviceAck(int timeout) { }
        //---------------------------------------------------------------------------

        public virtual bool DebuggingSearchSerialPortDevice() { return false; }


        //---------------------------------------------------------------------------
        //Add cycle query command
        public void CheckInformationListAdd()
        {
            checkInformationList.Clear();
            checkInformationList.Add(status);
        }
        public void LoopQuery()
        {
            checkInformationListIndexData = (char[])checkInformationList[checkInformationListIndex++];
            if (checkInformationListIndex >= checkInformationList.Count)
            {
                checkInformationListIndex = 0;
            }
            switch (checkInformationListIndex)
            {
                case 0:
                    StatusCmd();
                    break;
                
                default:
                    break;

            }
        }

        //---------------------------------------------------------------------------
        //User instruction sending and receiving processing
        public void UserTaskCmd()
        {
            string error = "error";
            WriteTxFrame(CommCmd.UserTask);
            WaiteForDeviceAck(ComTimeout);
            AcceptPrintInformation();
            chargeErrorCodeList.Clear();
            string seceiveDataString = new string(receiveDataBuffer);
            string[] statusProcessing = seceiveDataString.Split(CHAR_NEWLINE);
            string[] status = statusProcessing[0].Split(STRING_DELIM);
            Debug.WriteLine("User", seceiveDataString);
            if (status.Length >= 2)
            {
                if (status[1].Equals(error))
                {
                    ChargeErrorCodeProcessing(statusProcessing);
                }
                else
                {
                    TerminalForm.terminalForm.ChargeErrorCodeUIUpdate();
                    TaskCmd();
                }
            }
            
        }

        //
        public void ChargeErrorCodeProcessing(String[] data)
        {
            chargeErrorCodeList.Clear();
            for (int i = 0; i < data.Length - 1; i++)
            {
                string[] errorCode = data[0].Split(STRING_DELIM);
                chargeErrorCodeList.Add(errorCode[2]);
            }
            TerminalForm.terminalForm.ChargeErrorCodeUIUpdate();
        }


        //---------------------------------------------------------------------------
        // Debug mode user sending instructions and processing
        public void UserDebuggingTaskCmd()
        {
            WriteTxFrame(CommCmd.UserTask);
            WaiteForDeviceAck(ComTimeout);
            AcceptPrintInformation();
        }


        //---------------------------------------------------------------------------
        //Establish a secure link command to send and receive (hello)
        public bool TestLinkerCmd()
        {
            string hellos = "@hello";
            WriteTxFrame(CommCmd.TestLinker);
            WaiteForDeviceAck(ComTimeout);
            AcceptPrintInformation();
            string seceiveDataString = new string(receiveDataBuffer);
            string[] helloProcessing = seceiveDataString.Split(CHAR_NEWLINE);
            string[]  isHello = helloProcessing[0].Split(STRING_DELIM);
            if (isHello[0].ToLower().Equals(hellos))
            {
                int quantity = int.Parse(isHello[1]);
                TestLinkerProcessing(seceiveDataString, quantity);
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// Device online query instruction processing
        /// </summary>
        /// <param name="data">instruction</param>
        /// <param name="quantity">Device Quantity</param>
        public void TestLinkerProcessing(String data, int quantity)
        {
            /*
             * Examples of received instructions
             * @hello 2\n
             * hello SL0 CM1620  AP1.0.0.0  BT1.0.0.0  HW1.0.0.0\n
             * hello SL1 CM1620  AP1.0.0.0  BT1.0.0.0  HW1.0.0.0\n\r
             */
            deviceName.Clear();
            AP.Clear();
            BT.Clear();
            HW.Clear();
            string[] dataProcessing = data.Split(CHAR_NEWLINE);
            for (int i = 1; i <= quantity; i++)
            {
                string[] content = dataProcessing[i].Split(STRING_DELIM);
                deviceName.Add(content[1]);
                AP.Add(content[2]);
                BT.Add(content[3]);
                HW.Add(content[4]);
            }
            TerminalForm.terminalForm.DeviceConsistencyUIUpdate();
        }


        //---------------------------------------------------------------------------
        //Establish a secure link command to send and receive （logout）
        public bool LoganDviceCmd()
        {
            WriteTxFrame(CommCmd.LoganDevice);
            WaiteForDeviceAck(ComTimeout);
            AcceptPrintInformation();
            string seceiveDataString = new string(receiveDataBuffer); 
            string[] loganProcessing = seceiveDataString.Split(CHAR_NEWLINE);
            string[] logan = loganProcessing[0].Split(STRING_DELIM);
            int quantity = int.Parse(logan[1]);
            bool islogan = true;
            for (int i = 1; i <= quantity; i++)
            {
                string[] dataProcessing = loganProcessing[i].Split(STRING_DELIM);
                if (!dataProcessing[1].ToLower().Equals("ok"))
                {
                    islogan = false;
                    break;
                }
            }
            if (islogan)
            {
                CheckInformationListAdd();
                return true;
            }
            else return false;

        }


        //---------------------------------------------------------------------------
        //Task query command sending and receiving processing （task）
        public void TaskCmd()
        {
            WriteTxFrame(CommCmd.Task);
            WaiteForDeviceAck(ComTimeout);
            AcceptPrintInformation();
            string seceiveDataString = new string(receiveDataBuffer);
            if (seceiveDataString.Split(STRING_DELIM)[0].ToLower().Equals("@task"))
            {
                TaskDataProcessing(seceiveDataString);
            }
        }




        /// <summary>
        ///Receiving task query instruction data processing
        /// </summary>
        /// <param name="data">Received instructions</param>
        public void TaskDataProcessing(String data)
        {

            /*Examples of received instructions
             * @task 2\n
             * SL0 charge lipo  4.20V  12S  20000mAh  15.0A  BLN\n
             * SL1 charge lipo  4.20V  12S  20000mAh  15.0A  BLN\n\r
             */

            string[] dataProcessing = data.Split(CHAR_NEWLINE);
            for (int i = 1; i < dataProcessing.Length - 1; i++)
            {
                string[] basicInformation = dataProcessing[i].Split(STRING_DELIM);
                userdeviceNameList.Add(basicInformation[0]);
                userTaskTypeList.Add(basicInformation[1]);
                userBatteryTypeList.Add(basicInformation[2]);
                userVoltageList.Add(basicInformation[3]);
                userCellsList.Add(basicInformation[4]);
                userBatteryCapacityList.Add(basicInformation[5]);
                userCurrentList.Add(basicInformation[6]);
                userBalanceList.Add(basicInformation[7]);
            }

            TerminalForm.terminalForm.TaskUIUpdate();
        }


        //---------------------------------------------------------------------------
        //Status query command sending and receiving processing  (status)
        public void StatusCmd()
        {
            string isStatus = "@status";
            WriteTxFrame(CommCmd.Status);
            WaiteForDeviceAck(ComTimeout);
            AcceptPrintInformation();
            string seceiveDataString = new string(receiveDataBuffer);
            string[] statusProcessing = seceiveDataString.Split(CHAR_NEWLINE);
            string[] status = statusProcessing[0].Split(STRING_DELIM);
            if (status[0].ToLower().Equals(isStatus))
            {
                quantityModule = int.Parse(status[1]);
                StatusDataProcessing(statusProcessing, quantityModule);
            }
        }

        /// <summary>
        /// Receive status query instruction data processing
        /// </summary>
        /// <param name="dataProcessing">instruction</param>
        /// <param name="quantityModule">Device Quantity</param>
        public void StatusDataProcessing(String[] dataProcessing, int quantityModule)
        {
            moduleNameList.Clear();
            inputVoltageList.Clear();
            outputVoltageList.Clear();
            temperatureList.Clear();
            battGOList.Clear();
            batteryPercentage.Clear();
            balanceList.Clear();
            errorCodeList.Clear();
            currentTaskTypeList.Clear();
            taskCurrentList.Clear();
            inputPowerList.Clear();
            actualOutputCurrentList.Clear();
            actualOutputCurrent = 0;
            currentCapacityList.Clear();
            taskTimeList.Clear();
            balancePortVoltageList.Clear();
            balanceInternalResistanceList.Clear();
            string[] basicInformation = new string[] { };
            string[] taskRealTimeInformation = new string[] { };


            /* 
             * Examples of received instructions
             * @status 5\n
             * SL0 32.0V  24.0V  56C N 85% BVR 000 charging\n
             * 20.0A 400W 19.8A 15200mAh  00:00:30\n
             * 3.785 3.785 3.785 3.785 3.785 3.785 3.785 3.785 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0\n
             * 3.6 3.6 3.6 3.6 3.6 3.6 3.6 3.6 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0\n
             * SL1 32.0V  24.0V  56C N 100% BV 000 charging\n
             * 10.0A 400W 9.9A 15200mAh  00:00:30\n
             * 3.785 3.785 3.785 3.785 3.785 3.785 3.785 3.785 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0\n
             * 
             * 
             * @status 3\n
             * SL0 32.0V  24.0V  56C N 85% UBL 000 ConstCurChging\n
             * 40.0A 1000W 39.9A 15200mAh  00:00:30\n
             * SL1 32.0V  24.0V  56C N 85% UBL 000 ParallelChging\n
             * SL2 32.0V  24.0V  56C N 85% UBL 000 ParallelChging\n\r
             * 
            */
            for (int j = 1; j < dataProcessing.Length - 1; j++)
            {
                
                basicInformation = dataProcessing[j].Split(STRING_DELIM);
                moduleNameList.Add(basicInformation[0]);
                inputVoltageList.Add(basicInformation[1]);
                outputVoltageList.Add(basicInformation[2]);
                temperatureList.Add(basicInformation[3]);
                battGOList.Add(basicInformation[4]);
                batteryPercentage.Add(basicInformation[5]);
                balanceList.Add(basicInformation[6]);
                errorCodeList.Add(basicInformation[7]);
                currentTaskTypeList.Add(basicInformation[8]);

                if (!basicInformation[8].ToLower().Equals("standby") && !basicInformation[8].ToLower().Equals("abnormal") && !basicInformation[8].ToLower().Equals("parallelchging"))
                {

                    taskRealTimeInformation = dataProcessing[j + 1].Split(STRING_DELIM);
                    string actualOutputCurrents = taskRealTimeInformation[2].Substring(0, taskRealTimeInformation[2].Length - 1);
                    string currentCapacity = taskRealTimeInformation[3].Substring(0, taskRealTimeInformation[3].Length - 3);
                    taskCurrentList.Add(taskRealTimeInformation[0]);
                    inputPowerList.Add(taskRealTimeInformation[1]);
                    actualOutputCurrentList.Add(double.Parse(actualOutputCurrents));
                    //actualOutputCurrent = double.Parse(actualOutputCurrents);
                    currentCapacityList.Add(int.Parse(currentCapacity));
                    taskTimeList.Add(taskRealTimeInformation[4]);

                    switch (basicInformation[6])
                    {
                        case "BV":
                            balancePortVoltageList.Add(dataProcessing[j + 2].Split(STRING_DELIM));
                            AddInternalResistanceList();
                            j += 2;
                            break;
                        case "BVR":
                            balancePortVoltageList.Add(dataProcessing[j + 2].Split(STRING_DELIM));
                            balanceInternalResistanceList.Add(dataProcessing[j + 3].Split(STRING_DELIM));
                            j += 3;
                            break;
                        case "UBL":
                            if (j == 1)
                            {
                                j++;
                            }
                            AddPortVoltageAndInternalResistanceList();
                            break;
                        default:
                            break;
                    }

                }else if (basicInformation[8].ToLower().Equals("parallelchging"))
                {
                    inputPowerList.Add("0W");
                    currentCapacityList.Add(0);
                    actualOutputCurrentList.Add(0.0);
                    switch (basicInformation[6])
                    {
                        case "BV":
                            balancePortVoltageList.Add(dataProcessing[j + 1].Split(STRING_DELIM));
                            AddInternalResistanceList();
                            j += 2;
                            break;
                        case "BVR":
                            balancePortVoltageList.Add(dataProcessing[j + 1].Split(STRING_DELIM));
                            balanceInternalResistanceList.Add(dataProcessing[j + 2].Split(STRING_DELIM));
                            j += 3;
                            break;
                        case "UBL":
                            AddPortVoltageAndInternalResistanceList();
                            break;
                        default:
                            break;
                    }
                }
                else
                {
                    inputPowerList.Add("0W");
                    actualOutputCurrentList.Add(0.0);
                    currentCapacityList.Add(0);
                    switch (basicInformation[6])
                    {
                        case "BV":
                            balancePortVoltageList.Add(dataProcessing[j + 1].Split(STRING_DELIM));
                            AddInternalResistanceList();
                            j += 1;
                            break;
                        default:
                            AddPortVoltageAndInternalResistanceList();
                            break;
                    }
                }
       
            }

            TerminalForm.terminalForm.StatusUIUpdate();
            if(VoltageCalibrationForm.voltageCalibrationForm != null)
            {
                VoltageCalibrationForm.voltageCalibrationForm.VoltageUIUpdate();
            }
            
        }

        //Fill in empty data
        public void AddInternalResistanceList()
        {
            List<string> internalResistanceList = new List<string>();
            string[] internalResistance = new string[] { };
            for (int i = 0; i < 16; i++)
            {
                internalResistanceList.Add("0.00");
            }
            internalResistance = internalResistanceList.ToArray();
            balanceInternalResistanceList.Add(internalResistance);
        }
        //Fill in empty data
        public void AddPortVoltageAndInternalResistanceList()
        {
            List<string> balancceList = new List<string>();
            string[] balancce = new string[] { };
            for (int i = 0; i < 16; i++)
            {
                balancceList.Add("0.00");
            }
            balancce = balancceList.ToArray();
            balancePortVoltageList.Add(balancce);
            AddInternalResistanceList();

        }


        //---------------------------------------------------------------------------
        // Stop command sending and receiving processing (stop)
        public void StopCmd()
        {
            WriteTxFrame(CommCmd.Stop);
            WaiteForDeviceAck(ComTimeout);
            AcceptPrintInformation();
        }


        //---------------------------------------------------------------------------
        // Error recovery command sending and receiving processing (recover)
        public void RecoverCmd()
        {
            WriteTxFrame(CommCmd.Recover);
            WaiteForDeviceAck(ComTimeout);
            AcceptPrintInformation();
        }


        //---------------------------------------------------------------------------
        // Help command sending and receiving processing (help)
        public void HelpCmd()
        {
            WriteTxFrame(CommCmd.Help);
            WaiteForDeviceAck(ComTimeout);
            AcceptPrintInformation();
        }


        //---------------------------------------------------------------------------
        //Terminate the communication link sending and receiving processing (logout)
        public void LogoutCmd()
        {
            WriteTxFrame(CommCmd.Logout);
            WaiteForDeviceAck(ComTimeout);
            AcceptPrintInformation();
        }

        public void RebootCmd()
        {
            WriteTxFrame(CommCmd.Reboot);
            WaiteForDeviceAck(ComTimeout);
            AcceptPrintInformation();
        }


        //---------------------------------------------------------------------------
        //Voltage calibration command processing and sending (calibration)
        public void VoltageCalibrationCmd()
        {
            WriteTxFrame(CommCmd.UserTask);
            WaiteForDeviceAck(600);
            AcceptPrintInformation();
            if(receiveDataBuffer.Length > 1)
            {
                string seceiveDataString = new string(receiveDataBuffer);
                string[] statusProcessing = seceiveDataString.Split(CHAR_NEWLINE);
                string[] status = statusProcessing[0].Split(STRING_DELIM);
                string prompt = status[1];
                VoltageCalibrationForm.voltageCalibrationForm.PromptLabelUIUpdate(prompt);
            }
            


        }


        //---------------------------------------------------------------------------
        // Command judgment processing
        protected void WriteTxFrame(CommCmd cmd)
        {

            curCmd = cmd;
            switch (cmd)
            {
                
                case CommCmd.TestLinker:    
                    sendDataBuffer = hello;
                    sendDataBufferLength = hello.Length;
                    SendPrintInformation();
                    break;

                case CommCmd.LoganDevice:    
                    sendDataBuffer = login;
                    sendDataBufferLength = login.Length;
                    SendPrintInformation();
                    break;

                case CommCmd.Task:    
                    sendDataBuffer = task;
                    sendDataBufferLength = task.Length;
                    SendPrintInformation();
                    break;

                case CommCmd.Status:    
                    sendDataBuffer = status;
                    sendDataBufferLength = status.Length;
                    SendPrintInformation();
                    break;

                case CommCmd.Stop:    
                    sendDataBuffer = stop;
                    sendDataBufferLength = stop.Length;
                    SendPrintInformation();
                    break;

                case CommCmd.UserTask:   
                    sendDataBuffer = userTaskData;
                    sendDataBufferLength = userTaskData.Length;
                    SendPrintInformation();
                    break;

                case CommCmd.Recover:    
                    sendDataBuffer = recover;
                    sendDataBufferLength = recover.Length;
                    SendPrintInformation();
                    break;

                case CommCmd.Help:    
                    sendDataBuffer = help;
                    sendDataBufferLength = help.Length;
                    SendPrintInformation();
                    break;

                case CommCmd.Logout:    
                    sendDataBuffer = logout;
                    sendDataBufferLength = logout.Length;
                    SendPrintInformation();
                    break;
                case CommCmd.Reboot:
                    sendDataBuffer = reboot;
                    sendDataBufferLength = reboot.Length;
                    SendPrintInformation();
                    break;

                default:
                    sendDataBufferLength = 0;
                    break;
            }
            if (sendDataBufferLength != 0) SendDataFrame();
        }

        //---------------------------------------------------------------------------
        protected void SendDataFrame()
        {
            WriteMssage();
        }


        //---------------------------------------------------------------------------
        //Update and debug receiving instruction printing
        private void AcceptPrintInformation()
        {
            string seceiveDataString = new string(receiveDataBuffer);
            if (seceiveDataString.Length != 0)
            {
                printInformationData = seceiveDataString.Substring(0, seceiveDataString.Length - 1);
                TerminalForm.terminalForm.PrintInformationUIUpdate();
            }
            
        }

        //Update and debug send instruction print
        private void SendPrintInformation()
        {
            string sendDataString = new string(sendDataBuffer);
            printInformationData = sendDataString.Substring(0, sendDataString.Length - 1);
            TerminalForm.terminalForm.PrintInformationUIUpdate();
        }

    }
}
