﻿using System;
using System.Diagnostics;
using System.Resources;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CM1620_Terminal
{
    public partial class TerminalForm : Form
    {

        public static TerminalForm terminalForm;
        ResourceManager resourceManager = null;
        AutoSizeFormClass autoSizeFormClass = new AutoSizeFormClass();

        public TerminalForm()
        {

            //Obtain system language and modify software language
            string localLanguage = System.Globalization.CultureInfo.CurrentUICulture.Name;
            if (localLanguage.Contains("zh-CN"))
            {
                //Set language to Chinese
                resourceManager = new ResourceManager(typeof(ResourceCN));
                InitializeComponent();
                InitializeUI();
            }
            else
            {
                //Set language to English
                resourceManager = new ResourceManager(typeof(ResourceEN));
                InitializeComponent();
                InitializeUI();

            }

            CheckForIllegalCrossThreadCalls = false;
            terminalForm = this;
            InitializeDisplayAreaUIUpdate();

        }

        private void form_Load(object sender, EventArgs e)
        {
            autoSizeFormClass.controllInitializeSize(this);
        }

        private void form_SizeChanged(object sender, EventArgs e)
        {
            autoSizeFormClass.controlAutoSize(this);
        }



        enum TaskType
        {
            taskLogin,
            tyCheck,
            typeCheckInformation,
            tyUpdate,
            tyRename,
        };

        private TaskType nextTaskType = TaskType.typeCheckInformation;
        private TaskType taskType = TaskType.typeCheckInformation;

        //Create a delegate
        private delegate void MyDelegate(TaskType type);
        MyDelegate myDelegate = null;
        IAsyncResult asyncResult = null;

        private Stopwatch sw = new Stopwatch();
        private bool isDeviceConnect = false;
        private bool isLogan = false;
        private SmartLink smartLink = null;
        private UartLink uartLink = new UartLink(200);

        private delegate void PrintInformationUI();
        PrintInformationUI printInformationUI = null;

        private string STRING_PREFIX = "#";
        private string STRING_ADJUST = "#adjust ";
        private string STRING_SUFFIX = "\n\r";
        const string STRING_DELIM = " ";
        private int moduleDataGridViewIndex = 0;
        private int quantity = 0;
        private int quantitys = 0;

        private int module = 0;
        bool isModule = false;
        bool isInconsistentDeviceVersion = false;



        //Initialize the interface
        private void InitializeUI()
        {
            this.chargeErrorCodeLabel.Text = "";
            this.deviceConsistencyLabel.Text = "";
            this.displayGroupBox.Text = resourceManager.GetString("displayGroupBox.Text");
            this.siteGroupBox.Text = resourceManager.GetString("siteGroupBox.Text");
            this.label1.Text = resourceManager.GetString("setTaskLabel.Text");
            this.label2.Text = resourceManager.GetString("typeOfBatteryLabel.Text");
            this.label6.Text = resourceManager.GetString("useBalancePortLabel.Text");
            this.label8.Text = resourceManager.GetString("batteryCellLabel.Text");
            this.label3.Text = resourceManager.GetString("setVoltageLabel.Text");
            this.label7.Text = resourceManager.GetString("setCapacityLabel.Text");
            this.label9.Text = resourceManager.GetString("setCurrentLabel.Text");
            this.stopTaskButton.Text = resourceManager.GetString("stopButton.Text");
            this.modifyTaskButton.Text = resourceManager.GetString("modifyCurrentButton.Text");
            this.startTaskButton.Text = resourceManager.GetString("startButton.Text");
            this.connectButton.Text = resourceManager.GetString("connectDeviceButton.Text");
            this.sendOutButton.Text = resourceManager.GetString("sendButton.Text");
            this.debuggingGroupBox.Text = resourceManager.GetString("DebugModeGroupBox.Text");
            this.testLinkerButton.Text = resourceManager.GetString("equipmentOnlineQueryButton.Text");
            this.loganDviceButton.Text = resourceManager.GetString("establishAAecureLinkButton.Text");
            this.logoutButton.Text = resourceManager.GetString("check-outButton.Text");
            this.helpButton.Text = resourceManager.GetString("helpQueryButton.Text");
            this.recoverButton.Text = resourceManager.GetString("errorRecoveryCommandButton.Text");
            this.systemParameterButton.Text = resourceManager.GetString("systemParameterQueryButton.Text");
            this.taskButton.Text = resourceManager.GetString("taskQueryButton.Text");
            this.statusButton.Text = resourceManager.GetString("allSlaveStatusQueryButton.Text");
            this.stopButton.Text = resourceManager.GetString("stopButton.Text");
            this.smartBatteryButton.Text = resourceManager.GetString("smartBatteryQueryButton.Text");
            this.voltageCalibrationButton.Text = resourceManager.GetString("voltageCalibrationButton.Text");
            DataGridViewTextBoxColumn columns = new DataGridViewTextBoxColumn();
            DataGridViewTextBoxColumn columns1 = new DataGridViewTextBoxColumn();
            DataGridViewTextBoxColumn columns2 = new DataGridViewTextBoxColumn();
            DataGridViewTextBoxColumn columns3 = new DataGridViewTextBoxColumn();
            DataGridViewTextBoxColumn columns4 = new DataGridViewTextBoxColumn();
            DataGridViewTextBoxColumn columns5 = new DataGridViewTextBoxColumn();
            DataGridViewTextBoxColumn columns6 = new DataGridViewTextBoxColumn();
            columns.HeaderText = resourceManager.GetString("moduleName");
            columns1.HeaderText = resourceManager.GetString("inputVoltage");
            columns2.HeaderText = resourceManager.GetString("inputPower");
            columns3.HeaderText = resourceManager.GetString("outputVoltage");
            columns4.HeaderText = resourceManager.GetString("outputCurrent");
            columns5.HeaderText = resourceManager.GetString("temperature");
            columns6.HeaderText = resourceManager.GetString("workingStatus");
            this.ModuleDataGridView.Columns.Add(columns);
            this.ModuleDataGridView.Columns.Add(columns1);
            this.ModuleDataGridView.Columns.Add(columns2);
            this.ModuleDataGridView.Columns.Add(columns3);
            this.ModuleDataGridView.Columns.Add(columns4);
            this.ModuleDataGridView.Columns.Add(columns5);
            this.ModuleDataGridView.Columns.Add(columns6);
        }



        //Call asynchronously
        private void AsyncThread(TaskType type)
        {
            try
            {
                sw.Restart();

                if (!isDeviceConnect) SearchDevice();             //Search sevice
                if (!isLogan) Logon();
                if (type == TaskType.typeCheckInformation)
                {
                    smartLink.LoopQuery();                        //Cycle query command

                }

                isDeviceConnect = true;
                isLogan = true;
                ConnectUIUpdate();
            }
            catch (Exception e)
            {
                isDeviceConnect = false;
                isLogan = false;
                ConnectUIUpdate();
            }
            finally
            {
                sw.Stop();
            }

        }


        //Search device
        private void SearchDevice()
        {
            smartLink = uartLink;
            if (!smartLink.SearchSerialPortDevice())
            {
                //PrintInformationUIUpdates(resourceManager.GetString("can'tConnectToDevice"));
                throw new Exception("Connect the device failed.");

            }


        }



        private void Logon()
        {
            if (!smartLink.LoganDvice()) throw new Exception("Logan");
        }


        private void Completed(IAsyncResult result)
        {

        }


        //Device consistency UI
        public void DeviceConsistencyUIUpdate()
        {
            try
            {
                isInconsistentDeviceVersion = false;
                deviceConsistencyLabel.Text = "";
                if (smartLink.deviceName.Count > 1)
                {

                    for (int i = 0; i < smartLink.deviceName.Count; i++)
                    {
                        if (i + 1 < smartLink.deviceName.Count)
                        {
                            if (smartLink.deviceName[i].Equals(smartLink.deviceName[i + 1]))
                            {
                                if (smartLink.AP[i].Equals(smartLink.AP[i + 1]))
                                {


                                }
                                else
                                {
                                    isInconsistentDeviceVersion = true;
                                    deviceConsistencyLabel.Text = resourceManager.GetString("inconsistentDeviceVersion.Text");
                                    break;
                                }
                            }
                            else
                            {
                                isInconsistentDeviceVersion = true;
                                deviceConsistencyLabel.Text = resourceManager.GetString("deviceEquipment.Text");
                                break;
                            }
                        }
                    }

                }
            }
            catch (Exception e)
            {
            }


        }


        //Button event disconnected
        private void ConnectButton_Click(object sender, EventArgs e)
        {
            if (!isLogan)
            {
                moduleDataGridViewIndex = 0;
                quantity = 0;
                quantitys = 0;
                ModuleDataGridView.Rows.Clear();
                batteryDataGridView.Rows.Clear();
                timer1.Start();
            }
            else
            {
                smartLink.Close();
                timer1.Stop();
                isDeviceConnect = false;
                isLogan = false;
                ConnectUIUpdate();
            }

        }


        //Connected device timer
        private void timer1_Tick(object sender, EventArgs e)
        {
            if (asyncResult == null || asyncResult.IsCompleted)
            {
                myDelegate = new MyDelegate(AsyncThread);
                asyncResult = myDelegate.BeginInvoke(taskType = nextTaskType, new AsyncCallback(Completed), null);
                nextTaskType = TaskType.typeCheckInformation;
            }
        }


        // Update connection and disconnection  UI
        private void ConnectUIUpdate()
        {

            Task taskUI = new Task(() =>
            {
                try
                {
                    _ = this.Invoke((MethodInvoker)(() =>                             //Operation UI in child thread
                    {
                        if (isLogan)
                        {

                            this.connectButton.Text = resourceManager.GetString("disconnectButton.Text");
                        }
                        else
                        {
                            ModuleDataGridView.Rows.Clear();
                            batteryDataGridView.Rows.Clear();
                            quantity = 0;
                            quantitys = 0;
                            moduleDataGridViewIndex = 0;
                            timer1.Stop();
                            InitializeDisplayAreaUIUpdate();
                            smartLink = null;
                            PrintInformationUIUpdates(resourceManager.GetString("disconnectDevice.Text"));
                            this.connectButton.Text = resourceManager.GetString("connectDeviceButton.Text");
                        }

                    }));
                }
                catch (Exception e)
                {
                }
            });
            taskUI.Start();
        }


        //Debug printing information user interface update
        public void PrintInformationUIUpdate()
        {
            try
            {
                if (smartLink != null)
                {
                    if (!string.IsNullOrEmpty(smartLink.printInformationData))
                    {
                        informationLabel.AppendText(smartLink.printInformationData);
                        smartLink.printInformationData = null;

                    }
                }
            }
            catch (Exception e)
            {

            }
            
        }


        // Update task UI
        public void TaskUIUpdate()
        {
            batteryTypeLabel.Text = smartLink.userBatteryTypeList[0];
            cellsLabel.Text = smartLink.userCellsList[0];
            batteryCapacityLabel.Text = smartLink.userBatteryCapacityList[0];
        }


        //Update Charge Error UI
        public void ChargeErrorCodeUIUpdate()
        {
            if (smartLink.chargeErrorCodeList.Count > 0)
            {
                //for (int i = 0; i < smartLink.chargeErrorCodeList.Count; i++)
                //{
                string errorCode = smartLink.chargeErrorCodeList[0];
                switch (errorCode)
                {
                    case "101":
                        chargeErrorCodeLabel.Text = resourceManager.GetString("error101");
                        break;
                    case "102":
                        chargeErrorCodeLabel.Text = resourceManager.GetString("error102");
                        break;
                    case "103":
                        chargeErrorCodeLabel.Text = resourceManager.GetString("error103");
                        break;
                    case "104":
                        chargeErrorCodeLabel.Text = resourceManager.GetString("error104");
                        break;
                    case "105":
                        chargeErrorCodeLabel.Text = resourceManager.GetString("error105");
                        break;
                    case "106":
                        chargeErrorCodeLabel.Text = resourceManager.GetString("error106");
                        break;
                    case "107":
                        chargeErrorCodeLabel.Text = resourceManager.GetString("error107");
                        break;
                    default:
                        chargeErrorCodeLabel.Text = "";
                        break;
                }
                //}
            }
            else
            {
                chargeErrorCodeLabel.Text = "";
            }

        }



        //Update status UI
        public void StatusUIUpdate()
        {

            Task statusUI = new Task(() =>
            {
                try
                {
                    _ = this.Invoke((MethodInvoker)(() =>
                    {

                        double actualOutputCurrent = 0;
                        int currentCapacity = 0;

                        taskStatusLabel.Text = smartLink.currentTaskTypeList[0];
                        percentageLabel.Text = smartLink.batteryPercentage[0];


                        //Determine the  Status Error code
                        //for (int i = 0; i < smartLink.errorCodeList.Count; i++)
                        //{
                        string errorCode = smartLink.errorCodeList[0];
                        switch (errorCode)
                        {


                            case "151":
                                errorCodeLabel.Text = "151:ecParallel_SlxNum";
                                break;
                            case "152":
                                errorCodeLabel.Text = "152:ecParallel_Balance";
                                break;
                            case "153":
                                errorCodeLabel.Text = "153:ecParallel_OutputVoltage";
                                break;
                            case "154":
                                errorCodeLabel.Text = "154:ecParallel_TaskCurrent";
                                break;
                            case "155":
                                errorCodeLabel.Text = "155:ecParallel_StartError";
                                break;
                            case "156":
                                errorCodeLabel.Text = "156:ecParallel_LinkError";
                                break;
                            case "201":
                                errorCodeLabel.Text = resourceManager.GetString("error201");
                                break;
                            case "202":
                                errorCodeLabel.Text = resourceManager.GetString("error202");
                                break;
                            case "203":
                                errorCodeLabel.Text = resourceManager.GetString("error203");
                                break;
                            case "204":
                                errorCodeLabel.Text = resourceManager.GetString("error204");
                                break;
                            case "205":
                                errorCodeLabel.Text = resourceManager.GetString("error205");
                                break;
                            case "206":
                                errorCodeLabel.Text = resourceManager.GetString("error206");
                                break;
                            case "207":
                                errorCodeLabel.Text = resourceManager.GetString("error207");
                                break;
                            case "208":
                                errorCodeLabel.Text = resourceManager.GetString("error208");
                                break;
                            case "209":
                                errorCodeLabel.Text = resourceManager.GetString("error209");
                                break;
                            case "210":
                                errorCodeLabel.Text = resourceManager.GetString("error210");
                                break;
                            case "211":
                                errorCodeLabel.Text = resourceManager.GetString("error211");
                                break;
                            case "212":
                                errorCodeLabel.Text = resourceManager.GetString("error212");
                                break;
                            case "213":
                                errorCodeLabel.Text = "InOutDiffOverErr";
                                break;
                            case "301":
                                errorCodeLabel.Text = resourceManager.GetString("error301");
                                break;
                            case "302":
                                errorCodeLabel.Text = resourceManager.GetString("error302");
                                break;
                            case "303":
                                errorCodeLabel.Text = resourceManager.GetString("error303");
                                break;
                            case "304":
                                errorCodeLabel.Text = resourceManager.GetString("error304");
                                break;
                            case "305":
                                errorCodeLabel.Text = resourceManager.GetString("error305");
                                break;
                            case "306":
                                errorCodeLabel.Text = resourceManager.GetString("error306");
                                break;
                            case "307":
                                errorCodeLabel.Text = resourceManager.GetString("error307");
                                break;
                            case "308":
                                errorCodeLabel.Text = resourceManager.GetString("error308");
                                break;
                            case "309":
                                errorCodeLabel.Text = resourceManager.GetString("error309");
                                break;
                            case "310":
                                errorCodeLabel.Text = resourceManager.GetString("error310");
                                break;
                            case "311":
                                errorCodeLabel.Text = resourceManager.GetString("error311");
                                break;
                            case "312":
                                errorCodeLabel.Text = resourceManager.GetString("error312");
                                break;
                            case "313":
                                errorCodeLabel.Text = resourceManager.GetString("error313");
                                break;
                            case "401":
                                errorCodeLabel.Text = "401:ecTestError_bCalError";
                                break;
                            case "402":
                                errorCodeLabel.Text = "402:ecTestError_bBattGoLinkError";
                                break;
                            case "403":
                                errorCodeLabel.Text = "403:ecTestError_ADCRefError";
                                break;
                            case "404":
                                errorCodeLabel.Text = "404:ecTestError_TempError";
                                break;
                            case "405":
                                errorCodeLabel.Text = "405:ecTestError_CurrentRefError";
                                break;
                            case "406":
                                errorCodeLabel.Text = "406ecTestError_FanError";
                                break;
                            case "407":
                                errorCodeLabel.Text = "407:ecTestError_RevError";
                                break;
                            case "408":
                                errorCodeLabel.Text = "ecTestError_BattConnectError";
                                break;
                            case "409":
                                errorCodeLabel.Text = "ecTestError_InputVoltageLowError";
                                break;
                            case "410":
                                errorCodeLabel.Text = "ecTestError_InputVoltageHiError";
                                break;
                            case "411":
                                errorCodeLabel.Text = "ecTestError_DischargeError";
                                break;
                            case "412":
                                errorCodeLabel.Text = "ecTestError_BoostVolLowError";
                                break;
                            case "413":
                                errorCodeLabel.Text = "ecTestError_BoostVolHiError";
                                break;
                            case "414":
                                errorCodeLabel.Text = "eecTestError_BuckVolLowError";
                                break;
                            case "415":
                                errorCodeLabel.Text = "ecTestError_BuckVolHiError";
                                break;
                            case "416":
                                errorCodeLabel.Text = "ecTestError_bFactoryTimeError";
                                break;
                            case "417":
                                errorCodeLabel.Text = "ecTestError_BuckUnstableError";
                                break;
                            case "418":
                                errorCodeLabel.Text = "ecTestError_BoostUnstableError";
                                break;
                            case "419":
                                errorCodeLabel.Text = "ecTestError_AmpVoltError";
                                break;
                            case "420":
                                errorCodeLabel.Text = "ecTestError_ExtVoltError";
                                break;
                            case "421":
                                errorCodeLabel.Text = "ecTestError_VS45VoltError";
                                break;
                            case "422":
                                errorCodeLabel.Text = "ecTestError_FBIPWMError";
                                break;
                            case "423":
                                errorCodeLabel.Text = "ecTestError_BalancePortError";
                                break;
                            case "500":
                                errorCodeLabel.Text = "eSlaveCommTimeOut";
                                break;
                            case "501":
                                errorCodeLabel.Text = "eSlaveStartWorkTimeOut ";
                                break;
                            case "900":
                                errorCodeLabel.Text = "ecError_Slx";
                                break;
                            case "999":
                                errorCodeLabel.Text = "ecError_NoSupport";
                                break;
                            case "1000":
                                errorCodeLabel.Text = "ecError_Max";
                                break;

                            default:
                                errorCodeLabel.Text = "";
                                break;
                        }
                        //}


                        //Change the set current according to the number of devices
                        if (smartLink.quantityModule > 1)
                        {
                            isModule = false;
                            if (module != smartLink.quantityModule)
                            {
                                module = smartLink.quantityModule;
                                setCurrentComboBox.Items.Clear();
                                setCurrentComboBox.Text = "20A";
                                int endCurrent = 17 * smartLink.quantityModule - 1;                    //Current formula
                                for (double i = 20; i <= endCurrent; i++)
                                {
                                    setCurrentComboBox.Items.Add(i.ToString("f1") + "A");
                                }
                            }
                        }
                        else
                        {
                            if (!isModule)
                            {
                                isModule = true;
                                setCurrentComboBox.Items.Clear();
                                setCurrentComboBox.Text = "20A";
                                for (double i = 1; i <= 20; i++)
                                {
                                    setCurrentComboBox.Items.Add(i.ToString("f1") + "A");
                                }
                            }

                        }


                        //Show updated data
                        for (int i = 0; i < smartLink.quantityModule; i++)
                        {
                            actualOutputCurrent = actualOutputCurrent + smartLink.actualOutputCurrentList[i];
                            currentCapacity = currentCapacity + smartLink.currentCapacityList[i];
                        }

                        taskCurrentLabel.Text = actualOutputCurrent.ToString("0.0") + "A";
                        currentCapacityLabel.Text = currentCapacity + "mAh";


                        if (!smartLink.currentTaskTypeList[0].ToLower().Equals("standby") && !smartLink.currentTaskTypeList[0].ToLower().Equals("abnormal"))
                        {

                            timeLabel.Text = smartLink.taskTimeList[0];
                            startTaskButton.Enabled = false;
                            setTaskTypeComboBox.Enabled = false;
                            setBatteryTpyeComboBox.Enabled = false;
                            setBalanceComboBox.Enabled = false;
                            setCellsComboBox.Enabled = false;
                            setVoltageComboBox.Enabled = false;
                            setCapacityTextBox.Enabled = false;
                            voltageCalibrationButton.Enabled = false;
                        }
                        else
                        {
                            if (isInconsistentDeviceVersion)
                            {
                                startTaskButton.Enabled = false;
                                setTaskTypeComboBox.Enabled = false;
                                setBatteryTpyeComboBox.Enabled = false;
                                setBalanceComboBox.Enabled = false;
                                setCellsComboBox.Enabled = false;
                                setVoltageComboBox.Enabled = false;
                                setCapacityTextBox.Enabled = false;
                                voltageCalibrationButton.Enabled = false;
                                setCurrentComboBox.Enabled = false;
                                modifyTaskButton.Enabled = false;
                                stopTaskButton.Enabled = false;
                            }
                            else
                            {
                                setCurrentComboBox.Enabled = true;
                                modifyTaskButton.Enabled = true;
                                stopTaskButton.Enabled = true;
                                voltageCalibrationButton.Enabled = true;
                                startTaskButton.Enabled = true;
                                setTaskTypeComboBox.Enabled = true;
                                setBatteryTpyeComboBox.Enabled = true;
                                setBalanceComboBox.Enabled = true;
                                setCellsComboBox.Enabled = true;
                                setVoltageComboBox.Enabled = true;
                                setCapacityTextBox.Enabled = true;
                            }

                        }

                        if (quantity < smartLink.quantityModule)    //The number of devices increases, and ModuleDataGridView increases
                        {
                            ModuleDataGridView.Rows.Clear();
                            quantity = smartLink.quantityModule;
                            for (int i = 0; i < smartLink.quantityModule; i++)
                            {
                                int index = this.ModuleDataGridView.Rows.Add();
                                ModuleDataGridView.Rows[index].Cells[0].Value = smartLink.moduleNameList[i];
                                ModuleDataGridView.Rows[index].Cells[1].Value = smartLink.inputVoltageList[i];

                                ModuleDataGridView.Rows[index].Cells[2].Value = smartLink.inputPowerList[i];

                                ModuleDataGridView.Rows[index].Cells[3].Value = smartLink.outputVoltageList[i];

                                ModuleDataGridView.Rows[index].Cells[4].Value = smartLink.actualOutputCurrentList[i] + "A";

                                ModuleDataGridView.Rows[index].Cells[5].Value = smartLink.temperatureList[i];
                                ModuleDataGridView.Rows[index].Cells[6].Value = smartLink.currentTaskTypeList[i];

                            }

                        }
                        else if (quantity == smartLink.quantityModule)   //The number of devices remains unchanged, and the ModuleDataGridView remains unchanged
                        {
                            for (int i = 0; i < smartLink.quantityModule; i++)
                            {
                                ModuleDataGridView.Rows[i].Cells[0].Value = smartLink.moduleNameList[i];
                                ModuleDataGridView.Rows[i].Cells[1].Value = smartLink.inputVoltageList[i];
                                ModuleDataGridView.Rows[i].Cells[2].Value = smartLink.inputPowerList[i];
                                ModuleDataGridView.Rows[i].Cells[3].Value = smartLink.outputVoltageList[i];

                                ModuleDataGridView.Rows[i].Cells[4].Value = smartLink.actualOutputCurrentList[i] + "A";

                                ModuleDataGridView.Rows[i].Cells[5].Value = smartLink.temperatureList[i];
                                ModuleDataGridView.Rows[i].Cells[6].Value = smartLink.currentTaskTypeList[i];

                            }

                        }
                        else if (quantity > smartLink.quantityModule) //The number of devices is reduced, and the ModuleDataGridView is reduced
                        {

                            for (int i = 1; i <= quantity - smartLink.quantityModule; i++)
                            {
                                quantity = smartLink.quantityModule;
                                DataGridViewRow row = ModuleDataGridView.Rows[ModuleDataGridView.Rows.Count - 1];
                                ModuleDataGridView.Rows.Remove(row);
                            }
                        }


                        if (quantitys < smartLink.quantityModule)
                        {
                            batteryDataGridView.Rows.Clear();
                            quantitys = smartLink.quantityModule;
                            for (int i = 0; i < smartLink.quantityModule; i++)
                            {
                                int indexV = this.batteryDataGridView.Rows.Add();
                                string[] balancePortVoltageArray = (string[])smartLink.balancePortVoltageList[i];
                                for (int j = 0; j < 16; j++)
                                {
                                    batteryDataGridView.Rows[indexV].Cells[j].Value = balancePortVoltageArray[j] + "V";
                                }

                                int indexI = this.batteryDataGridView.Rows.Add();

                                string[] balanceInternalResistanceArray = (string[])smartLink.balanceInternalResistanceList[i];
                                for (int j = 0; j < 16; j++)
                                {
                                    batteryDataGridView.Rows[indexI].Cells[j].Value = balanceInternalResistanceArray[j] + "mR";
                                }

                            }
                        }
                        else if (quantitys == smartLink.quantityModule)
                        {

                            for (int i = 0; i < quantitys; i++)
                            {

                                string[] balancePortVoltageArray = (string[])smartLink.balancePortVoltageList[i];
                                for (int j = 0; j < 16; j++)
                                {
                                    if (i > 0)
                                    {
                                        batteryDataGridView.Rows[i * 2].Cells[j].Value = balancePortVoltageArray[j] + "V";
                                    }
                                    else
                                    {
                                        batteryDataGridView.Rows[i].Cells[j].Value = balancePortVoltageArray[j] + "V";
                                    }

                                }

                                string[] balanceInternalResistanceArray = (string[])smartLink.balanceInternalResistanceList[i];
                                for (int j = 0; j < 16; j++)
                                {
                                    if (i > 0)
                                    {
                                        batteryDataGridView.Rows[i * 2 - 1].Cells[j].Value = balanceInternalResistanceArray[j] + "mR";
                                    }
                                    else
                                    {
                                        batteryDataGridView.Rows[i + 1].Cells[j].Value = balanceInternalResistanceArray[j] + "mR";
                                    }

                                }

                            }
                        }
                        else if (quantitys > smartLink.quantityModule)
                        {
                            for (int i = 1; i <= quantitys - smartLink.quantityModule; i++)
                            {
                                quantitys = smartLink.quantityModule;
                                DataGridViewRow row = batteryDataGridView.Rows[batteryDataGridView.Rows.Count - 1];
                                batteryDataGridView.Rows.Remove(row);
                                DataGridViewRow rows = batteryDataGridView.Rows[batteryDataGridView.Rows.Count - 1];
                                batteryDataGridView.Rows.Remove(rows);
                            }
                        }



                    }));
                }
                catch (Exception e)
                {
                }
            });
            statusUI.Start();

        }


        //Initialize display area UI
        public void InitializeDisplayAreaUIUpdate()
        {
            batteryTypeLabel.Text = "--";
            cellsLabel.Text = "--S";
            batteryCapacityLabel.Text = "--mAh";
            taskCurrentLabel.Text = "--A";
            currentCapacityLabel.Text = "--mAh";
            timeLabel.Text = "00:00:00";
            chargeErrorCodeLabel.Text = "";
        }


        private void timer2_Tick(object sender, EventArgs e)
        {
            printInformationUI = new PrintInformationUI(PrintInformationUIUpdate);
            Invoke(printInformationUI);
        }



        //Stop task Button event
        private void StopTaskButton_Click(object sender, EventArgs e)
        {
            if (smartLink != null)
            {
                smartLink.StopCmd();
                InitializeDisplayAreaUIUpdate();
            }

        }


        //Modify task Button event
        private void ModifyTaskButton_Click(object sender, EventArgs e)
        {
            if (smartLink != null)
            {
                string getDate = STRING_ADJUST + setCurrentComboBox.Text + STRING_SUFFIX;
                smartLink.userTaskData = getDate.ToCharArray();
                smartLink.UserTaskCmd();
            }

        }


        //Start task Button event
        private void startTaskButton_Click(object sender, EventArgs e)
        {
            string balanceData;
            if (smartLink != null)
            {
                if (setBalanceComboBox.Text.Equals("true"))
                {
                    balanceData = "BLN";
                }
                else
                {
                    balanceData = "UBL";
                }

                string getDate = STRING_PREFIX
                    + setTaskTypeComboBox.Text + STRING_DELIM
                    + setBatteryTpyeComboBox.Text + STRING_DELIM
                    + setVoltageComboBox.Text + STRING_DELIM
                    + setCellsComboBox.Text + STRING_DELIM
                    + setCapacityTextBox.Text + STRING_DELIM
                    + setCurrentComboBox.Text + STRING_DELIM
                    + balanceData + STRING_SUFFIX;
                smartLink.userTaskData = getDate.ToCharArray();
                smartLink.UserTaskCmd();
            }

        }

        //Send Out Button event
        private void SendOutButton_Click(object sender, EventArgs e)
        {
            string getDate = STRING_PREFIX
                    + debuggingTextBox.Text
                    + STRING_SUFFIX;

            if (smartLink != null)
            {
                smartLink.userTaskData = getDate.ToCharArray();
                smartLink.UserDebuggingTaskCmd();
            }
            else
            {
                if (IsDebuggingSearchSerialPortDevice())
                {
                    smartLink.userTaskData = getDate.ToCharArray();
                    smartLink.UserDebuggingTaskCmd();
                }

            }

        }


        /// <summary>
        /// Update the content of debugging print instruction
        /// </summary>
        /// <param name="data">Printed instructions</param>
        public void PrintInformationUIUpdates(string data)
        {
            this.informationLabel.Focus();
            this.informationLabel.Select(this.informationLabel.TextLength, 0);
            this.informationLabel.ScrollToCaret();
            informationLabel.AppendText(data);
        }

        private void timer3_Tick(object sender, EventArgs e)
        {

        }


        //Device online query Button event
        private void testLinkerButton_Click(object sender, EventArgs e)
        {

            if (smartLink != null)
            {

                smartLink.TestLinkerCmd();
            }
            else
            {
                if (IsDebuggingSearchSerialPortDevice()) smartLink.TestLinkerCmd();


            }

        }

        //Establish a secure link Button event
        private void loganDviceButton_Click(object sender, EventArgs e)
        {
            if (smartLink != null)
            {

                smartLink.LoganDviceCmd();
            }
            else
            {
                if (IsDebuggingSearchSerialPortDevice()) smartLink.LoganDviceCmd();

            }
        }

        //Terminate the communication link Button event
        private void logoutButton_Click(object sender, EventArgs e)
        {

            if (smartLink != null)
            {

                smartLink.LogoutCmd();
            }
            else
            {
                if (IsDebuggingSearchSerialPortDevice()) smartLink.LogoutCmd();


            }


        }

        //Task query Button event
        private void taskButton_Click(object sender, EventArgs e)
        {

            if (smartLink != null)
            {

                smartLink.TaskCmd();
            }
            else
            {
                if (IsDebuggingSearchSerialPortDevice()) smartLink.TaskCmd();


            }

        }


        //Status query Button event
        private void statusButton_Click(object sender, EventArgs e)
        {

            if (smartLink != null)
            {

                smartLink.StatusCmd();
            }
            else
            {
                if (IsDebuggingSearchSerialPortDevice()) smartLink.StatusCmd();


            }
        }


        //Stop pause Button event
        private void StopButton_Click(object sender, EventArgs e)
        {

            if (smartLink != null)
            {

                smartLink.StopCmd();
            }
            else
            {
                if (IsDebuggingSearchSerialPortDevice()) smartLink.StopCmd();


            }

        }

        //Determine if the device is connected
        private bool IsDebuggingSearchSerialPortDevice()
        {
            smartLink = uartLink;
            if (!smartLink.DebuggingSearchSerialPortDevice())
            {
                smartLink = null;
                PrintInformationUIUpdates(resourceManager.GetString("can'tConnectToDevice"));
                return false;
            }
            else
            {
                Debug.WriteLine("Open successfully");
                return true;
            }
        }


        //Help query Button event
        private void helpButton_Click(object sender, EventArgs e)
        {
            if (smartLink != null)
            {

                smartLink.HelpCmd();
            }
            else
            {
                if (IsDebuggingSearchSerialPortDevice()) smartLink.HelpCmd();

            }
        }


        //Clear error Button event
        private void recoverButton_Click(object sender, EventArgs e)
        {
            if (smartLink != null)
            {

                smartLink.RecoverCmd();
            }
            else
            {
                if (IsDebuggingSearchSerialPortDevice()) smartLink.RecoverCmd();


            }
        }

        private void button14_Click(object sender, EventArgs e)
        {

        }

        private void setVoltageComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {

        }


        //Update the set voltage of the corresponding battery type
        private void SetBatteryTpyeComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            string setBatteryTpye = setBatteryTpyeComboBox.Text;
            switch (setBatteryTpye)
            {
                case "LiPo":
                    setVoltageComboBox.Items.Clear();
                    setVoltageComboBox.Text = "4.20V";
                    for (double i = 4.10; i <= 4.30; i += 0.01)
                    {

                        setVoltageComboBox.Items.Add(i.ToString("f2") + "V");
                    }
                    break;
                case "LiHv":
                    setVoltageComboBox.Items.Clear();
                    setVoltageComboBox.Text = "4.30V";
                    for (double i = 4.25; i <= 4.45; i += 0.01)
                    {
                        setVoltageComboBox.Items.Add(i.ToString("f2") + "V");
                    }
                    break;
                case "LiFe":
                    setVoltageComboBox.Items.Clear();
                    setVoltageComboBox.Text = "3.65V";
                    for (double i = 3.55; i <= 3.75; i += 0.01)
                    {
                        setVoltageComboBox.Items.Add(i.ToString("f2") + "V");
                    }
                    break;
                default:
                    break;
            }
        }


        //Voltage calibration Button event
        private void voltageCalibration_Click(object sender, EventArgs e)
        {
            //if (smartLink != null)
            //{
            VoltageCalibrationForm voltageCalibrationForm = new VoltageCalibrationForm(smartLink);
            voltageCalibrationForm.ShowDialog();
            //}

        }

        private void rebootButton_Click(object sender, EventArgs e)
        {

            if (smartLink != null)
            {

                smartLink.RebootCmd();
            }
            else
            {
                if (IsDebuggingSearchSerialPortDevice()) smartLink.RebootCmd();


            }

        }
    }
}
