﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO.Ports;
using System.Text;
using System.Threading;

namespace CM1620_Terminal
{
    class UartLink : SmartLink
    {
        SerialPort mySerialPort = new SerialPort();
        const char CHAR_PREFIX = '@';
        const char CHAR_SUFFIX = '\r';

        //---------------------------------------------------------------------------
        protected override bool Open(string port)
        {
            Close();
            // Pier name
            mySerialPort.PortName = port;
            // Baud rate 256000
            mySerialPort.BaudRate = 256000;
            //mySerialPort.BaudRate = 115200;
            // 8 data bits
            mySerialPort.DataBits = 8;
            // 1 stop bit
            mySerialPort.StopBits = StopBits.One;
            // No parity
            mySerialPort.Parity = Parity.None;
            // Read serial port timeout time is 1000ms
            mySerialPort.ReadTimeout = 1000;
            //  Read serial port timeout time is 1000ms
            mySerialPort.WriteTimeout = 1000;
            // Read buffer size
            mySerialPort.ReadBufferSize = 4096;
            // Output buffer size
            mySerialPort.WriteBufferSize = 4096;

            mySerialPort.Open();

            return true;
        }
        //---------------------------------------------------------------------------
        public override void Close()
        {
            if (mySerialPort.IsOpen) mySerialPort.Close();
        }
        //---------------------------------------------------------------------------
        //Get communication timeout
        public UartLink(int timeout)
            : base(timeout)
        {

        }
        //---------------------------------------------------------------------------
        //Search Serial Port Device
        public override bool SearchSerialPortDevice()
        {
            // Get a list of serial port names.
            String[] serialPortArray = SerialPort.GetPortNames();
            if (serialPortArray != null && serialPortArray.Length != 0)
            {
                //Open the serial port and send the #hello command
                Array.Sort<String>(serialPortArray);
                foreach (String port in serialPortArray)
                {
                    try
                    {
                        if (Open(port))
                        {
                           if (TestLinkerCmd()) return true;
                        }
                    }
                    catch { }
                    Close();
                }
            }
            return false;
        }


        ////Debugging Search Serial Port Device
        public override bool DebuggingSearchSerialPortDevice()
        {
            // Get a list of serial port names.
            String[] serialPortArray = SerialPort.GetPortNames();
            if (serialPortArray != null && serialPortArray.Length != 0)
            {
                //Serial port
                Array.Sort<String>(serialPortArray);
                foreach (String port in serialPortArray)
                {
                    try
                    {
                        if (Open(port))
                        {
                            return true;
                        }
                    }
                    catch { }
                    Close();
                }
            }
            return false;
        }



        public override bool LoganDvice()
        {
            try
            {
                if (LoganDviceCmd()) return true;
            }
            catch { }
            return false;

        }

        //---------------------------------------------------------------------------
        //Send command to CM1620
        protected override void WriteMssage()
        {
            try
            {
                if ((mySerialPort != null) && (mySerialPort.IsOpen))
                {
                    SmLnkPkgMsg = false;
                    commState = CommState.csStart;
                    string data = new string(sendDataBuffer);
                    mySerialPort.Write(sendDataBuffer, 0, sendDataBufferLength);
                }
            }
            catch (Exception e)
            {
                Debug.WriteLine("异常", e);
            }
            
        }
        //---------------------------------------------------------------------------


        private void serialPort_DataReceivedEventHandler(object sender, SerialDataReceivedEventArgs e)
        {
            try
            {

                List<char> buffer = new List<char>(4096);
                int n = mySerialPort.BytesToRead;
                char[] buf = new char[n];
                mySerialPort.Read(buf, 0, n);
                buffer.AddRange(buf);
                receiveDataBuffer = buf;
            }
            catch (TimeoutException)
            {

            }

        }

        /*
         *Waiting for CM620 to confirm and receive reply command
         * "timeout"  timeout communication timeout
         */
        protected override void WaiteForDeviceAck(int timeout)
        {
            byte RxTryCount = 0;
            int totalSleepTime = 0;
            Thread.Sleep(200);
            try
            {
                receiveDataBuffer = new char[] { };
                int n = mySerialPort.BytesToRead;
                char[] buffer = new char[n];
                mySerialPort.Read(buffer, 0, n);
                //Filter illegal instructions
                if (buffer.Length > 2)
                {
                    if (buffer[0].Equals(CHAR_PREFIX) && buffer[n - 1].Equals(CHAR_SUFFIX)) receiveDataBuffer = buffer;
                }
                
            }
            catch (TimeoutException)
            {
                //Communication timeout handling
                totalSleepTime += mySerialPort.ReadTimeout;
                if (timeout <= totalSleepTime)
                {
                    if (++RxTryCount > 1) throw new Exception("Communication timeout.");
                    totalSleepTime = 0;
                    SendDataFrame();
                }
            }
            if (SmLnkPkgMsg) return;

        }
    }
}